import 'dart:io';
import 'package:recase/recase.dart';
import '../models/file_model.dart';
import '../samples/interface/sample_interface.dart';

class Structure {
  static Map<String, String> _paths = {
    'page': replaceAsExpected(path: 'lib/pages/'),
    'widget': replaceAsExpected(path: 'lib/widgets/'),
    'model': replaceAsExpected(path: 'lib/data/models/'),
    'init': replaceAsExpected(path: 'lib/'),
    'route': replaceAsExpected(path: 'lib/routes/'),
    'repository': replaceAsExpected(path: 'lib/data/repositories/'),
    'provider': replaceAsExpected(path: 'lib/data/providers/'),
    'controller': replaceAsExpected(path: 'controllers/'),
    'view': replaceAsExpected(path: 'views/'),
  };

  static FileModel model(String name, String command, bool wrapperFolder,
      {String on}) {
    if (on != null) {
      // FileModel _fileModel = FileModel(
      //   name: '',
      //   path: Structure.getPathWithName(
      //     'init',
      //     ReCase('').snakeCase,
      //     createWithWrappedFolder: false,
      //   ),
      //   commandName: 'init',
      // );

      Directory current = Directory('./lib');
      final list = current.listSync(recursive: true, followLinks: false);
      final contains = list.firstWhere((element) => element.path.contains(on),
          orElse: () => null);

      return FileModel(
        name: name,
        path: Structure.getPathWithName(
          contains.path,
          ReCase(name).snakeCase,
          createWithWrappedFolder: true,
        ),
        commandName: command,
      );
    }
    return FileModel(
      name: name,
      path: Structure.getPathWithName(
        _paths[command],
        ReCase(name).snakeCase,
        createWithWrappedFolder: wrapperFolder,
      ),
      commandName: command,
    );
  }

  static String replaceAsExpected({String path, String replaceChar}) {
    if (path.contains('\\')) {
      if (Platform.isLinux || Platform.isMacOS) {
        return path.replaceAll('\\', '/');
      } else {
        return path;
      }
    } else if (path.contains('/')) {
      if (Platform.isWindows) {
        return path.replaceAll('/', '\\\\');
      } else {
        return path;
      }
    } else {
      return path;
    }
  }

  static String getPathWithName(String firstPath, String secondPath,
      {bool createWithWrappedFolder = false}) {
    String betweenPaths;
    if (Platform.isWindows) {
      betweenPaths = "\\\\";
    } else if (Platform.isMacOS || Platform.isLinux) {
      betweenPaths = "/";
    }
    if (betweenPaths.isNotEmpty) {
      if (createWithWrappedFolder) {
        return firstPath +
            betweenPaths +
            secondPath +
            betweenPaths +
            secondPath;
      } else {
        return firstPath + betweenPaths + secondPath;
      }
    }
    return null;
  }
}

Future<File> createFile({FileModel fileModel, Sample content}) async {
  var reCase = ReCase(fileModel.name);
  var _file = await File(fileModel.path + '_${fileModel.commandName}.dart')
      .create(recursive: true);
  await _file.writeAsString(content.file(reCase.pascalCase));

  print(
      'file created succesfully with name :${fileModel.name} at path: ${fileModel.path}');
  return _file;
}
